'use strict';

exports.__esModule = true;

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _Project = require('../Project');

var _Project2 = _interopRequireDefault(_Project);

var _Workspace = require('../Workspace');

var _Workspace2 = _interopRequireDefault(_Workspace);

var _logger = require('../utils/logger');

var logger = _interopRequireWildcard(_logger);

var _messages = require('../utils/messages');

var messages = _interopRequireWildcard(_messages);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function versionRangeToRangeType(versionRange) {
  if (versionRange.charAt(0) === '^') return '^';
  if (versionRange.charAt(0) === '~') return '~';
  return '';
}

/**
 * This function is used to update all the internal dependencies where you have an external source
 * bumping versions (a tool like bolt-releases for example).
 * It takes an object of packageNames and their new versions. updatePackageVersions will update all
 * internal versions of packages according to those new versions.
 * ie, a caret dep, will remain a caret dep and a pinned dep will remain pinned.
 *
 * It is up to the consumer to ensure that these new versions are not going to leave the repo in an
 * inconsistent state (internal deps leaving semver ranges).
 *
 * Note: we explicitly ignore all external dependencies passed and warn if they are.
 */

exports.default = async function updatePackageVersions(versions, opts = {}) {
  let cwd = opts.cwd || process.cwd();
  let project = await _Project2.default.init(cwd);
  let workspaces = await project.getWorkspaces();
  let { graph } = await project.getDependencyGraph(workspaces);
  let updatedPackages = new _set2.default();

  const internalDeps = (0, _keys2.default)(versions).filter(dep => graph.has(dep));
  const externalDeps = (0, _keys2.default)(versions).filter(dep => !graph.has(dep));

  if (externalDeps.length !== 0) {
    logger.warn(messages.externalDepsPassedToUpdatePackageVersions(externalDeps));
  }

  for (let workspace of workspaces) {
    let pkg = workspace.pkg;
    let promises = [];

    for (let depName of internalDeps) {
      const depRange = String(pkg.getDependencyVersionRange(depName));
      const depTypes = pkg.getDependencyTypes(depName);
      const rangeType = versionRangeToRangeType(depRange);
      const newDepRange = rangeType + versions[depName];

      if (depTypes.length === 0) continue;
      for (let depType of depTypes) {
        await pkg.setDependencyVersionRange(depName, depType, newDepRange);
      }
      updatedPackages.add(pkg.filePath);
    }
  }

  return (0, _from2.default)(updatedPackages);
};